<?php

require_once 'base/FpaeventomEntityBase.php';


/**
 * FpaeventomEntity.
 *
 * @package Entity
 * @author  alessandrobartels
 *
 */
class FpaeventomEntity extends FpaEventomEntityBase
{


//    private $_lista;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_dtPagamento;
    private $_tipoEvento;
    public $preservar;
 //   private $historico;


    /**
     * Construtor.
     *
     * @param integer    $id             Identificador.
     * @param integer    $nrOrg          Numero da Organizacao de Trabalho.
     * @param integer    $nrOrgPadrao    Numero da Organizacao Padrao.
     * @param TkDateTime $mesCompetencia Mes de competencia.
     * @param string     $dtPagamento    Data de pagamento.
     *
     * @author alessandrobartels
     */
    public function __construct(
        $id = null, $nrOrg = null, $nrOrgPadrao = null, TkDateTime $mesCompetencia = null, $dtPagamento = null
    ){
        parent::__construct($id);
        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;
        $this->_dtPagamento = $dtPagamento;

        $this->_tipoEvento = array(
                              self::IDTIPOEVENTO_PROVENTO => '1',
                              self::IDTIPOEVENTO_DESCONTO => '2',
                              self::IDTIPOEVENTO_ENCARGO  => '3',
                              self::IDTIPOEVENTO_TOTAL    => '4',
                             );
    }


    public function setMesCompetencia($mesCompetencia){
        $this->_mesCompetencia = $mesCompetencia;
    }


    public function setDataPagamento($dataPagamento){
        $this->_dtPagamento = $dataPagamento;
    }


    /**
     * Retorna a ordem do tipo de evento. Metodo usado para impressao do evento.
     *
     * @param string $ordem Posicao do array associativo (tipo do evento).
     *
     * @return string
     *
     * @author alessandrobartels - 13/10/2011
     */
    public function getOrdemTipoEvento($ordem) {
        return $this->_tipoEvento[$ordem];
    }


    /**
     * Inicializa lista de eventos.
     *
     * @param integer $nrOrg Numero da Organizacao.
     *
     * @return void
     *
     * @author alessandrobartels -
     *         alessandrobartels - 27/06/2011 - Implementacao do NRORG
     */
    private function _inicializaLista($nrOrg){
        $tmp = Fpaeventom::retornaEventos($nrOrg, $this->_nrOrgPadrao); // $tmp = self::getEventos($nrOrg, $this->_nrOrgPadrao);
        $this->preservar = array();
        foreach ($tmp as $tmp_evento) {
            //verifica se o evento foi parametrizado com prioridade de estorno e carrega lista de eventos que
            //devem ser preservados. Esta lista será usada no calculo para compor os valores a preservar no calculo de
            //complemento de líquido
            //leandroafonso 21/11/2013
            if($tmp_evento->historico->getNrpriorestorno() == 999){
                $this->preservar[]=$tmp_evento->getNreventom();
            }
            $tmp_evento->_mesCompetencia = $this->_mesCompetencia;
            $tmp_evento->_dtPagamento = $this->_dtPagamento;
            /* O evento deve ser indexado pelo seu numero  na lista. Se fosse indexado por indice sequencial o
             * retornaEvento teria que fazer um loop toda vez que fosse chamado.
             */

            $this->addItem($tmp_evento, $tmp_evento->getNreventom());
            /* $this->addItem($tmp_evento); */
        }
        unset($tmp);
    }


    /**
     * Retorna instancia do evento parametrizado.
     *
     * @param integer $nr_evento Numero do evento.
     * @param integer $nrOrg     Numero da Organizacao.
     *
     * @return FpaeventomEntity
     *
     * @author alessandrobartels -
     *         alessandrobartels - 27/06/2011 - Implementacao do NRORG
     */
    public function retornaEvento($nr_evento){
        /* @todo verificar o nrorg passado como parametro. Ja atribuido no construtor*/
        //$nrOrg = $this->_nrOrg;
        //$itens = $this->getItems();
        if (empty($this->items) === true) {
            $this->_inicializaLista($this->_nrOrg);
            //$itens = $this->getItems();
        }
        return $this->items[$nr_evento];
    }


    /*@todo verificar os metodos abaixo para retornar uma lista de eventos media
    		olhar como foi feito no calculo
    */


    /**
     * Inicializa lista de eventos de media.
     *
     * @return void
     *
     * @author alessandrobartels -
     *         alessandrobartels - 27/06/2011 - Implementacao do NRORG
     *         alessandrobartels - 29/11/2012 - Retirada Org do parametro do metodo. Ela eh variavel de instancia
     */
    private function _inicializaListaMedia(){
        $tmp = Fpaeventom::retornaListaEventosMedia($this->_nrOrg, $this->_nrOrgPadrao);
        foreach ($tmp as $tmp_evento) {
            /* O evento deve ser indexado pelo seu numero  na lista. Se fosse indexado por indice sequencial o
             * retornaListaEventoMedia teria que fazer um loop toda vez que fosse envocado.
             */
            $this->addItem($tmp_evento, $tmp_evento->getNreventom());
            /* $this->addItem($tmp_evento); */
        }
    }


    /**
     * Retorna lista dos numeros de eventos de media.
     *
     * @param integer $nrOrg Numero da Organizacao.
     *
     * @return string
     *
     * @author alessandrobartels -
     *         alessandrobartels - 27/06/2011 - Implementacao do NRORG
     *         alessandrobartels - 29/11/2012 - Retirada Org do parametro do metodo. Ela eh variavel de instancia
     */
    public function retornaListaEventoMedia(){

        $itens = $this->getItems();

        if (empty($itens) === true) {
            $this->_inicializaListaMedia();
            $itens = $this->getItems();
        }

        $lista='';
        foreach ($itens as $item) {
            $lista = $lista.$item->getNreventom().',';
        }

            $lista = substr($lista, 0, strlen($lista)-1);


        return $lista;
    }


    /**
     * Inicializa lista de eventos de faltas.
     *
     * @param integer $nrOrg Numero da Organizacao.
     *
     * @return void
     *
     * @author alessandrobartels -
     *         alessandrobartels - 27/06/2011 - Implementacao do NRORG
     */
    private function _inicializaListaFalta($nrOrg, $nrOrgPadrao){
        $tmp = Fpaeventom::retornaListaEventosFalta($nrOrg, $nrOrgPadrao);
        foreach ($tmp as $tmp_evento) {
            /* O evento deve ser indexado pelo seu numero  na lista. Se fosse indexado por indice sequencial o
             * retornaListaEventoFalta teria que fazer um loop toda vez que fosse envocado.
             */
            $this->addItem($tmp_evento, $tmp_evento->getNreventom());
            /* $this->addItem($tmp_evento); */
        }
    }


    /**
     * Retorna lista dos numeros de eventos de falta.
     *
     * @param integer $nrOrg Numero da Organizacao.
     *
     * @return string
     *
     * @author alessandrobartels -
     *         alessandrobartels - 27/06/2011 - Implementacao do NRORG
     */
    public function retornaListaEventoFalta($nrOrg, $nrOrgPadrao){

        $itens = $this->getItems();

        if (empty($itens) === true) {
            $this->_inicializaListaFalta($nrOrg, $nrOrgPadrao);
            $itens = $this->getItems();
            $lista='';
            foreach ($itens as $item) {
                $lista = $lista.$item->getNreventom().',';
            }

            $lista = substr($lista, 0, strlen($lista)-1);
        }
         //BORBA 11/11/2013 item resetado pois ja passa pelo metodo na preparção e a condição empty($itens) === true não satifaz na segunda vez.
        $this->resetItems();
        return $lista;
    }


    /**
     * Enter description here ...
     *
     * @param string $dtIniVigencia Data de inicio de vigencia.
     * @param string $idTipoEvento  Tipo do evento.
     *
     * @return boolean
     *
     * @author luistavares 11/08/2011
     */
    public function novoMestre($dtIniVigencia, $idTipoEvento){
        try{
            $evento = new self(null, $this->_nrOrg, $this->_nrOrgPadrao);

            $evento->setDtinivigencia($dtIniVigencia);
            $evento->setIdtipoevento($idTipoEvento);
            $evento->setNrorg($this->_nrOrg);
            $evento->persist();

            $nrEvento = $evento->getNreventom();
            unset($evento);
            return $nrEvento;

        }
        catch (Exception $e){
            throw new Exception('Erro ao criar um novo evento. Nao pode inserir novo mestre.<br/>'.$e->getMessage());
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer $nreventom Nr Evento.
     * @param integer $nrorgTrab Nr Org.
     *
     * @return boolean
     *
     * @author luistavares 11/08/2011
     */
    public function destroiMestre($nreventom, $nrorgTrab){
        try{
            $evento = Fpaeventom::retornaEvento($nreventom, $nrorgTrab);
            if (is_object($evento) === true) {
                /* Adiciona um item � lista, ap�s isso, o item � removido da lista
                 * e enviado para lista de itens removidos, lista essa que ir� apagar
                 * todos seus itens quando executar o persist(). */
                $this->addItem($evento, $nreventom);
                $this->removeItem($evento);
                $this->persist();

                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            tk_warning($e->getMessage());
            return false;
        }

    }


    /**
     * Conforme parametro no evento retorna a data de ocorrencia.
     *
     * @return TkDateTime|string
     *
     * @author alessandrobartels 25/08/2011
     */
    public function determinaDataOcorrencia(){

        $tipoData = $this->historico->getIddtocorrencia();
        switch ($tipoData) {
        case FpaeventohEntity::IDDTOCORRENCIA_COMPETENCIA:
            return $this->_mesCompetencia; /* TkDateTime */
        case FpaeventohEntity::IDDTOCORRENCIA_PAGAMENTO:
            return $this->_dtPagamento; /* string */
        }
    }

    /**
     * @return FpaeventomEntity
     */
    public function insereEventoMestre($nrorg, $idTipoEvento, $dtIniVigencia) {
        $this->setNrorg($nrorg);
        $this->setIdtipoevento($idTipoEvento);
        $this->setDtinivigencia($dtIniVigencia);
    }

    /**
     * Retorna array associativo com todos os eventos, sendo a chave o nreventom
     *
     * @param $nrorg
     * @param $nrorgpadrao
     *
     * @return array
     * @autor jorgefonseca
     */
    public static function getEventosArray($nrorg, $nrorgpadrao) {
        $lista = array();
        $eventos = self::getEventos($nrorg, $nrorgpadrao);
        /** @var $evento FpaeventomEntity */
        foreach($eventos as $evento) {
            $lista[$evento->getNreventom()] = $evento;
        }
        return $lista;
    }

    /**
     * @param $nrorg
     * @param $nrorgpadrao
     *
     * @return array
     */
    public static function getEventos($nrorg, $nrorgpadrao) {
        $eventos = self::getEntitiesByWhere('NRORG = ? OR NRORG = ?', array($nrorg, $nrorgpadrao));
        return $eventos;
    }

    public static function getEvento($nreventom) {
        $evento = FpaeventomEntity::getEntityByWhere('NREVENTOM = ?', array($nreventom));
        return $evento;
    }

    /**
     * @return FpaeventohEntity
     */
    public function getHistorico() {
        if (!$this->historico) {
            $this->historico = FpaeventohEntity::getUltimoHistorico($this->getNreventom());
        }
        return $this->historico;
    }

}